import pytest
import os
import sys
import numpy as np

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import gext
import gext.grassmann
import utils

SMALL = 1e-10
THRESHOLD = 1e-2

@pytest.mark.parametrize("datafile", ["urea.json", "glucose.json"])
def test_extrapolation(datafile):

    # load test data from json file
    data = utils.load_json(f"tests/{datafile}")
    nelectrons = data["nelectrons"]
    natoms = data["trajectory"].shape[1]
    nbasis = data["overlaps"].shape[1]
    nframes = data["trajectory"].shape[0]

    # amount of data we want to use for fitting
    n = 9
    assert n < nframes

    # initialize an extrapolator
    extrapolator = gext.Extrapolator(nelectrons, nbasis, natoms, n)

    # load data in the extrapolator up to index n - 1
    for (coords, coeff, overlap) in zip(data["trajectory"][:n],
            data["coefficients"][:n], data["overlaps"][:n]):
        extrapolator.load_data(coords, coeff, overlap)

    # check an extrapolation at index n
    guessed_density = extrapolator.guess(data["trajectory"][n], data["overlaps"][n])
    coeff = data["coefficients"][n][:, :nelectrons//2]
    density = coeff @ coeff.T

    assert np.linalg.norm(guessed_density - density, ord=np.inf) < THRESHOLD
    assert np.linalg.norm(guessed_density - density, ord=np.inf) \
          /np.linalg.norm(density, ord=np.inf) < THRESHOLD
